import json
from .utils import to_csv, to_json, to_pdf  ###
from langchain_openai import ChatOpenAI
from langchain.prompts import PromptTemplate
from langchain.chains import LLMChain
from langchain.output_parsers import PydanticOutputParser
from .models import MCQList ###



def generate_mcq(topic, num=1, llm=None, response_model=None , prompt_template=None, **kwargs):

    if response_model == None:
      parser = PydanticOutputParser(pydantic_object = MCQList)
      format_instructions = parser.get_format_instructions()
    else:
      parser = PydanticOutputParser(pydantic_object = response_model)
      format_instructions = parser.get_format_instructions()

    if prompt_template is None:
        prompt_template = """
        Generate {num} multiple-choice question (MCQ) based on the given topic and level.
        provide the question, four answer options, and the correct answer.

        Topic: {topic}
        """

    # Append the JSON format instruction line to the custom prompt template
    prompt_template += "\nThe response should be in JSON format. \n {format_instructions}"

    MCQ_prompt = PromptTemplate(
        input_variables=["num", "topic"],
        template=prompt_template,
        partial_variables={"format_instructions": format_instructions}
    )

    if llm:
        llm = llm
    else:
        llm = ChatOpenAI(model="gpt-3.5-turbo")

    # MCQ_chain = LLMChain(llm=llm, prompt=MCQ_prompt)
    MCQ_chain = MCQ_prompt | llm

    results = MCQ_chain.invoke(
        {"num": num, "topic": topic, **kwargs},
        # return_only_outputs=True

    )

    results = results.content
    structured_output = parser.parse(results)

    return structured_output




from .utils import to_csv, to_json, to_pdf
from .qna_engine import generate_mcq
from .content_engine import generate_lesson_plan

__all__ = ['generate_mcq', 'to_csv', 'to_json', 'to_pdf', 'generate_lesson_plan']


from typing import List
from langchain_core.pydantic_v1 import BaseModel

class MCQ(BaseModel):
    """A class representing a multiple choice question."""
    question: str
    options: List[str]
    correct_answer: str
    explanation: str
    
    def show(self):
        options_str = "\n".join(f"  {chr(65 + i)}. {option}" for i, option in enumerate(self.options))
        print(f"Question: {self.question}\nOptions:\n{options_str}\nCorrect Answer: {self.correct_answer}\nExplanation: {self.explanation}\n")


class MCQList(BaseModel):
    questions: List[MCQ]

    def show(self):
        print("MCQs:\n")
        for i, mcq in enumerate(self.questions):
            print(f"Question {i + 1}:")
            mcq.show()

class LessonPlan(BaseModel):
    """A class representing a lesson plan."""
    topic: str
    objectives: List[str]
    introduction: str
    content: str
    assessment: str
    conclusion: str

    def show(self):
        print(f"Topic: {self.topic}")
        print("Objectives:")
        for objective in self.objectives:
            print(f"- {objective}")
        print(f"Introduction: {self.introduction}")
        print(f"Content: {self.content}")
        print(f"Assessment: {self.assessment}")
        print(f"Conclusion: {self.conclusion}\n")


from langchain_openai import ChatOpenAI
from langchain.prompts import PromptTemplate
from langchain.chains import LLMChain
from langchain.output_parsers import PydanticOutputParser
from .models import LessonPlan

def generate_lesson_plan(topic, llm=None, response_model=None, prompt_template=None, **kwargs):
    if response_model == None:
        parser = PydanticOutputParser(pydantic_object=LessonPlan)
        format_instructions = parser.get_format_instructions()
    else:
        parser = PydanticOutputParser(pydantic_object=response_model)
        format_instructions = parser.get_format_instructions()

    if prompt_template is None:
        prompt_template = """
        Generate a comprehensive lesson plan for the given topic and duration.
        Include the following details in the lesson plan:
        - Objectives: List the learning objectives of the lesson.
        - Introduction: Provide an engaging introduction to the lesson.
        - Content Outline: Outline the main points or sections of the lecture content.
        - Assessment: Describe how the students' understanding will be assessed.
        - Conclusion: Summarize the key takeaways and provide a conclusion for the lesson.

        Topic: {topic}
        """

    # Append the JSON format instruction line to the custom prompt template
    prompt_template += "\nThe response should be in JSON format. \n {format_instructions}"

    lesson_plan_prompt = PromptTemplate(
        input_variables=["topic"],
        template=prompt_template,
        partial_variables={"format_instructions": format_instructions}
    )

    if llm:
        llm = llm
    else:
        llm = ChatOpenAI(model="gpt-3.5-turbo")

    lesson_plan_chain = lesson_plan_prompt | llm

    results = lesson_plan_chain.invoke(
        {"topic": topic, **kwargs},
    )
    results = results.content
    structured_output = parser.parse(results)
    return structured_output


from setuptools import setup, find_packages

setup(
    name="educhain",
    version="0.2.2",
    packages=find_packages(),
    install_requires=[
        "langchain",
        "langchain-community",
        "langchain-openai",
        "openai",
        "python-dotenv", 
        "pandas",
        "reportlab"
    ],
    author="Satvik Paramkusham",
    author_email="satvik@buildfastwithai.com",
    description="A Python package for generating educational content using Generative AI",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/satvik314/educhain",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



import pandas as pd
from .models import MCQList  ###
from reportlab.lib.pagesizes import letter
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, PageBreak
from reportlab.lib.styles import getSampleStyleSheet

def to_csv(quiz_data : MCQList, file_name):
    """
    Generate a CSV file from a Quiz object.

    Args:
    quiz_data (Quiz): Instance of the Quiz class containing a list of Question objects.
    file_name (str): Name of the CSV file to be created.
    """
    mcq_data = []

    for question in quiz_data.questions:
        mcq_data.append({
            'question': question.question,
            'option_1': question.options[0],
            'option_2': question.options[1],
            'option_3': question.options[2],
            'option_4': question.options[3],
            'correct_answer': question.correct_answer
        })

    df = pd.DataFrame(mcq_data)
    df.to_csv(file_name, index=False)
    

def to_json(quiz_data : MCQList, file_name=None):

    """
    Convert a list of Question objects to JSON and create a JSON file.

    Args:
    questions (list): List of Question objects.
    file_name (str): Name of the JSON file to be created.
    """
    data = [{"question": question.question, "options": question.options, "correct_answer": question.correct_answer} for question in quiz_data.questions]
    
    df = pd.DataFrame(data)
    
    if file_name:
        df.to_json(file_name, orient='records', indent=4)
        
    return data


def to_pdf(quiz_data : MCQList, file_name, heading=None, subheading=None):
    """
    Create a PDF file from a list of MCQ (Multiple Choice Questions).

    Args:
    questions (list): List of Question objects.
    file_name (str): Name of the PDF file to be created.
    heading (str): Heading for the PDF document. (optional)
    subheading (str): Subheading for the PDF document. (optional)
    """
    styles = getSampleStyleSheet()

    doc = SimpleDocTemplate(file_name, pagesize=letter)
    elements = []

    if heading:
        elements.append(Paragraph(heading, styles["Heading1"]))

    if subheading:
        elements.append(Paragraph(subheading, styles["Heading2"]))
        elements.append(Spacer(1, 12))

    for i, question in enumerate(quiz_data.questions, start=1):
        question_text = f"{i}. {question.question}"
        elements.append(Paragraph(question_text, styles["BodyText"]))

        for j, option in enumerate(question.options, start=97):
            option_text = f"{chr(j)}) {option}"
            elements.append(Paragraph(option_text, styles["BodyText"]))

        elements.append(Spacer(1, 12))

    elements.append(PageBreak())  # Add a page break before the answers
    elements.append(Paragraph("Answers", styles["Heading1"]))

    for i, question in enumerate(quiz_data.questions, start=1):
        correct_answer_text = f"{i}. {chr(question.options.index(question.correct_answer) + 97)}) {question.correct_answer}"
        elements.append(Paragraph(correct_answer_text, styles["BodyText"]))

    doc.build(elements)
